"""
pygitscrum argparse gestion
"""

import argparse
import sys


def compute_args():
    """
    check args and return them
    """
    my_parser = argparse.ArgumentParser(
        description="pygitscrum : masterize git!!!",
        epilog="""
        Full documentation at: <https://github.com/thib1984/pygitscrum>.
        Report bugs to <https://github.com/thib1984/pygitscrum/issues>.
        MIT Licence.
        Copyright (c) 2021 thib1984.
        This is free software: you are free to change and redistribute it.
        There is NO WARRANTY, to the extent permitted by law.
        Written by thib1984.""",
    )
    my_parser.add_argument(
        "-v",
        "--debug",
        action="store_true",
        help="full trace",
    )
    my_group = my_parser.add_mutually_exclusive_group(required=True)
    my_group.add_argument(
        "-d",
        "--daily",
        metavar="git_period",
        action="store",
        type=str,
        nargs="?",
        const="yesterday",
        help="list your commits since <git_period> (default : yesterday) from all branches",
    )
    my_group.add_argument(
        "-s",
        "--search",
        metavar="keyword",
        action="store",
        type=str,
        help="search in logs 'keyword' from all",
    )
    my_group.add_argument(
        "-c",
        "--check",
        action="store_true",
        help="fetch all, ask you action if a pull/push is available",
    )
    my_group.add_argument(
        "-w",
        "--wip",
        action="store_true",
        help="fetch all, print stashs, branches with push available, and number of no commit changed files ",
    )
    my_group.add_argument(
        "-p",
        "--prune",
        action="store_true",
        help="DRY RUN! : fetch all, print stash, and gone local branch",
    )
    my_group.add_argument(
        "-t",
        "--track",
        action="store_true",
        help="fetch all, track new branches, delete refs from inexisting remote, fetch all",
    )
    my_group.add_argument(
        "-V",
        "--version",
        action="store_true",
        help="display pygitscrum's version",
    )
    my_group.add_argument(
        "-u",
        "--update",
        action="store_true",
        help="update pygitscrum",
    )

    my_parser.add_argument(
        "to_path",
        metavar="to_path",
        type=str,
        nargs="?",
        default=".",
        help="the path to scan, defaut '.'",
    )

    # if no parameter
    if len(sys.argv) == 1:
        my_parser.print_help()
        sys.exit(0)

    args = my_parser.parse_args()
    return args



"""
--version scripts
"""

import pkg_resources


def version_pygitscrum():
    """
    entry point for --version
    """
    print(
        "version pygitscrum : "
        + pkg_resources.get_distribution("pygitscrum").version
    )



"""
scan anf gest repo scripts
"""

import glob
import os
from termcolor import colored
from pygitscrum.args import compute_args


def scan_directories():
    """
    entry point for --scan
    """
    pathname = compute_args().to_path + "/**/.git"
    files = glob.glob(pathname, recursive=True)
    files.sort()
    if (
        len(files) == 0
        and not compute_args().update
        and not compute_args().version
    ):
        print(colored("no found local repos!", "red"))
    return files


def absolute_path_without_git(directory):
    """
    return the absolute path of local git repo
    """
    return os.path.abspath(directory + "/..")


def print_repo_if_first(first, repo):
    """
    print the repo only if first == True
    """
    if first:
        first = False
        print(
            colored(
                absolute_path_without_git(repo),
                "green",
            )
        )
    return first



"""
pygitscrum init
"""

from pygitscrum.args import compute_args
from pygitscrum.scan import scan_directories
from pygitscrum.update import update_pygitscrum
from pygitscrum.version import version_pygitscrum
from pygitscrum.git_search import git_search
from pygitscrum.git_check import git_check
from pygitscrum.git_daily import git_daily
from pygitscrum.git_track import git_track
from pygitscrum.git_wip import git_wip
from pygitscrum.git_prune import git_prune


def pygitscrum():
    """
    pygitscrum entry point
    """
    args = compute_args()

    files = scan_directories()

    if args.version:
        version_pygitscrum()
    elif args.update:
        update_pygitscrum()
    elif args.track:
        git_track(files)
    elif args.check:
        git_check(files)
    elif args.daily:
        git_daily(files)
    elif args.search:
        git_search(files)
    elif args.wip:
        git_wip(files)
    elif args.prune:
        git_prune(files)



"""
--check scripts
"""

from termcolor import colored
from pygitscrum.git import (
    command_git_call,
    command_git_call_print,
    command_git_check,
    command_git_check_en,
    command_git_check_en_print,
)
from pygitscrum.args import compute_args
from pygitscrum.scan import absolute_path_without_git


def git_check(files):
    """
    entry point for --check
    """
    files_to_work = []
    for repo in files:
        if compute_args().debug:
            print(
                "debug : " + absolute_path_without_git(repo) + " ..."
            )

        ############################################
        # UPDATE + FETCH
        ############################################
        command_git_check_en_print(repo, ["remote", "update"], True)
        command_git_check_en_print(repo, ["fetch", "--all"], True)
        ############################################

        ############################################
        # GIT STATUS
        ############################################
        while "Your branch is up to date" not in command_git_check_en(
            repo, ["status"]
        ):
            print(colored(absolute_path_without_git(repo), "yellow"))
            print(
                colored(
                    command_git_check(repo, ["status"]),
                    "yellow",
                )
            )
            answer = input("p(ull)/P(ush)/s(how)/S(how all)/q(uit)? ")
            if answer == "p":
                command_git_call_print(repo, ["pull"], True)
            elif answer == "P":
                command_git_call_print(repo, ["push"], True)
            elif answer == "S":
                command_git_call(
                    repo, ["log", "-p", "HEAD..FETCH_HEAD"]
                )
                command_git_call(
                    repo, ["log", "-p", "FETCH_HEAD..HEAD"]
                )
            elif answer == "s":
                command_git_call(
                    repo,
                    [
                        "log",
                        "-p",
                        "--name-only",
                        "HEAD..FETCH_HEAD",
                    ],
                )
                command_git_call(
                    repo,
                    [
                        "log",
                        "-p",
                        "--name-only",
                        "FETCH_HEAD..HEAD",
                    ],
                )
            else:
                files_to_work.append(absolute_path_without_git(repo))
                break

    ############################################
    if len(files_to_work) > 0:
        print("")
        print(
            colored(
                "Folders that will be checked : \n"
                + "\n".join(files_to_work),
                "yellow",
            )
        )



"""
--prune scripts
"""

from termcolor import colored
from pygitscrum.git import (
    command_git_check_en_print,
    command_git_check,
)
from pygitscrum.scan import (
    absolute_path_without_git,
    print_repo_if_first,
)
from pygitscrum.args import compute_args


def git_prune(files):
    """
    entry point for --prune
    """
    for repo in files:
        if compute_args().debug:
            print(
                "debug : " + absolute_path_without_git(repo) + " ..."
            )

        ############################################
        # UPDATE + FETCH
        ############################################
        command_git_check_en_print(repo, ["remote", "update"], True)
        command_git_check_en_print(repo, ["fetch", "--all"], True)

        ############################################
        # GIT STATUS
        ############################################

        wip_stash = command_git_check(repo, ["stash", "list"])
        diff_branches = command_git_check(
            repo,
            [
                "for-each-ref",
                '--format="%(refname:short) %(upstream:track) (upstream:remotename)"'
                "refs/heads",
            ],
        )

        first = True
        if wip_stash != "":
            for line in wip_stash.split("\n"):
                if "stash" in line:
                    first = print_repo_if_first(first, repo)
                    print(colored("stash - " + line, "yellow"))
        if diff_branches != "":
            for line in diff_branches.split("\n"):
                if "[gone]" in line:
                    first = print_repo_if_first(first, repo)
                    print(
                        colored(
                            "gone branch - " + line,
                            "yellow",
                        )
                    )
